@@ -19,6 +19,7 @@ import (
 	"fmt"
 	"math/bits"
 	"net"
+	"strconv"
 	"strings"
 	"unsafe"
 
@@ -38,6 +39,7 @@ const (
 
 type LookupPrefix struct {
 	Prefix string
+	RD     string
 	LookupOption
 }
 
@@ -233,6 +235,54 @@ func (t *Table) GetLongerPrefixDestinations(key string) ([]*Destination, error)
 			mask = 8 - ones&0x7
 			div = ones >> 3
 		}
+		r.WalkPrefix(p, func(n *net.IPNet, v interface{}) bool {
+			if mask != 0 && n.IP[div]>>mask != p.IP[div]>>mask {
+				return true
+			}
+			l, _ := n.Mask.Size()
+
+			if ones > l {
+				return true
+			}
+			results = append(results, v.(*Destination))
+			return true
+		})
+	case bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN:
+		prefixRd, _, network, err := bgp.ParseVPNPrefix(key)
+		if err != nil {
+			return nil, err
+		}
+		ones, bits := network.Mask.Size()
+
+		r := critbitgo.NewNet()
+		for _, dst := range t.GetDestinations() {
+			var dstRD bgp.RouteDistinguisherInterface
+			switch t.routeFamily {
+			case bgp.RF_IPv4_VPN:
+				dstRD = dst.nlri.(*bgp.LabeledVPNIPAddrPrefix).RD
+			case bgp.RF_IPv6_VPN:
+				dstRD = dst.nlri.(*bgp.LabeledVPNIPv6AddrPrefix).RD
+			}
+
+			if prefixRd.String() != dstRD.String() {
+				continue
+			}
+
+			r.Add(nlriToIPNet(dst.nlri), dst)
+		}
+
+		p := &net.IPNet{
+			IP:   network.IP,
+			Mask: net.CIDRMask((ones>>3)<<3, bits),
+		}
+
+		mask := 0
+		div := 0
+		if ones%8 != 0 {
+			mask = 8 - ones&0x7
+			div = ones >> 3
+		}
+
 		r.WalkPrefix(p, func(n *net.IPNet, v interface{}) bool {
 			if mask != 0 && n.IP[div]>>mask != p.IP[div]>>mask {
 				return true
@@ -339,6 +389,20 @@ func (t *Table) tableKey(nlri bgp.AddrPrefixInterface) string {
 		copy(b, T.Prefix.To16())
 		b[16] = T.Length
 		return *(*string)(unsafe.Pointer(&b))
+	case *bgp.LabeledVPNIPAddrPrefix:
+		b := make([]byte, 13)
+		serializedRD, _ := T.RD.Serialize()
+		copy(b, serializedRD)
+		copy(b[8:12], T.Prefix.To4())
+		b[12] = T.Length
+		return *(*string)(unsafe.Pointer(&b))
+	case *bgp.LabeledVPNIPv6AddrPrefix:
+		b := make([]byte, 25)
+		serializedRD, _ := T.RD.Serialize()
+		copy(b, serializedRD)
+		copy(b[8:24], T.Prefix.To16())
+		b[24] = T.Length
+		return *(*string)(unsafe.Pointer(&b))
 	}
 	return nlri.String()
 }
@@ -403,20 +467,24 @@ func (t *Table) Select(option ...TableSelectOption) (*Table, error) {
 	if len(prefixes) != 0 {
 		switch t.routeFamily {
 		case bgp.RF_IPv4_UC, bgp.RF_IPv6_UC:
-			f := func(prefixStr string) bool {
+			f := func(prefixStr string) (bool, error) {
 				var nlri bgp.AddrPrefixInterface
+				var err error
 				if t.routeFamily == bgp.RF_IPv4_UC {
-					nlri, _ = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP, bgp.SAFI_UNICAST, prefixStr)
+					nlri, err = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP, bgp.SAFI_UNICAST, prefixStr)
 				} else {
-					nlri, _ = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP6, bgp.SAFI_UNICAST, prefixStr)
+					nlri, err = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP6, bgp.SAFI_UNICAST, prefixStr)
+				}
+				if err != nil {
+					return false, err
 				}
 				if dst := t.GetDestination(nlri); dst != nil {
 					if d := dst.Select(dOption); d != nil {
 						r.setDestination(d)
-						return true
+						return true, nil
 					}
 				}
-				return false
+				return false, nil
 			}
 
 			for _, p := range prefixes {
@@ -453,7 +521,11 @@ func (t *Table) Select(option ...TableSelectOption) (*Table, error) {
 							if err != nil {
 								return nil, err
 							}
-							if f(prefix.String()) {
+							ret, err := f(prefix.String())
+							if err != nil {
+								return nil, err
+							}
+							if ret {
 								break
 							}
 						}
@@ -462,6 +534,108 @@ func (t *Table) Select(option ...TableSelectOption) (*Table, error) {
 					}
 				}
 			}
+		case bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN:
+			f := func(prefixStr string) error {
+				var nlri bgp.AddrPrefixInterface
+				var err error
+
+				if t.routeFamily == bgp.RF_IPv4_VPN {
+					nlri, err = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP, bgp.SAFI_MPLS_VPN, prefixStr)
+				} else {
+					nlri, err = bgp.NewPrefixFromRouteFamily(bgp.AFI_IP6, bgp.SAFI_MPLS_VPN, prefixStr)
+				}
+				if err != nil {
+					return fmt.Errorf("failed to create prefix: %w", err)
+				}
+
+				if dst := t.GetDestination(nlri); dst != nil {
+					if d := dst.Select(dOption); d != nil {
+						r.setDestination(d)
+					}
+				}
+				return nil
+			}
+
+			for _, p := range prefixes {
+				switch p.LookupOption {
+				case LOOKUP_LONGER:
+					_, prefix, err := net.ParseCIDR(p.Prefix)
+					if err != nil {
+						return nil, err
+					}
+
+					if p.RD == "" {
+						for _, dst := range t.GetDestinations() {
+							tablePrefix := nlriToIPNet(dst.nlri)
+
+							if bgp.ContainsCIDR(prefix, tablePrefix) {
+								r.setDestination(dst)
+							}
+						}
+
+						return r, nil
+					}
+
+					ds, err := t.GetLongerPrefixDestinations(p.RD + ":" + p.Prefix)
+					if err != nil {
+						return nil, err
+					}
+
+					for _, dst := range ds {
+						if d := dst.Select(dOption); d != nil {
+							r.setDestination(d)
+						}
+					}
+				case LOOKUP_SHORTER:
+					addr, prefix, err := net.ParseCIDR(p.Prefix)
+					if err != nil {
+						return nil, err
+					}
+
+					if p.RD == "" {
+						for _, dst := range t.GetDestinations() {
+							tablePrefix := nlriToIPNet(dst.nlri)
+
+							if bgp.ContainsCIDR(tablePrefix, prefix) {
+								r.setDestination(dst)
+							}
+						}
+
+						return r, nil
+					}
+
+					rd, err := bgp.ParseRouteDistinguisher(p.RD)
+					if err != nil {
+						return nil, err
+					}
+
+					ones, _ := prefix.Mask.Size()
+					for i := ones; i >= 0; i-- {
+						_, prefix, _ := net.ParseCIDR(addr.String() + "/" + strconv.Itoa(i))
+
+						err := f(rd.String() + ":" + prefix.String())
+						if err != nil {
+							return nil, err
+						}
+					}
+				default:
+					if p.RD == "" {
+						for _, dst := range t.GetDestinations() {
+							net := nlriToIPNet(dst.nlri)
+							if net.String() == p.Prefix {
+								r.setDestination(dst)
+							}
+						}
+
+						return r, nil
+					}
+
+					err := f(p.RD + ":" + p.Prefix)
+					if err != nil {
+						return nil, err
+					}
+				}
+			}
 		case bgp.RF_EVPN:
 			for _, p := range prefixes {
 				// Uses LookupPrefix.Prefix as EVPN Route Type string
