@@ -20,6 +20,7 @@ import (
 	"context"
 	"fmt"
 	"io"
+	"math"
 	"net"
 	"reflect"
 	"regexp"
@@ -42,6 +43,9 @@ import (
 	"github.com/osrg/gobgp/v3/pkg/packet/bgp"
 )
 
+// Unlimited batch size by default
+const defaultListPathBatchSize = math.MaxUint64
+
 type server struct {
 	bgpServer  *BgpServer
 	grpcServer *grpc.Server
@@ -225,19 +229,38 @@ func getValidation(v map[*table.Path]*table.Validation, p *table.Path) *table.Va
 func (s *server) ListPath(r *api.ListPathRequest, stream api.GobgpApi_ListPathServer) error {
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
+	batchSize := r.BatchSize
+	if batchSize == 0 {
+		batchSize = defaultListPathBatchSize
+	}
 	l := make([]*api.Destination, 0)
+	send := func() error {
+		for _, d := range l {
+			if err := stream.Send(&api.ListPathResponse{Destination: d}); err != nil {
+				return err
+			}
+		}
+		return nil
+	}
+	var sendErr error
 	err := s.bgpServer.ListPath(ctx, r, func(d *api.Destination) {
-		l = append(l, d)
+		if uint64(len(l)) < batchSize {
+			l = append(l, d)
+			return
+		}
+		if sendErr = send(); sendErr != nil {
+			cancel()
+			return
+		}
+		l = l[:0]
 	})
+	if sendErr != nil {
+		return sendErr
+	}
 	if err != nil {
 		return err
 	}
-	for _, d := range l {
-		if err := stream.Send(&api.ListPathResponse{Destination: d}); err != nil {
-			break
-		}
-	}
-	return err
+	return send()
 }
 
 func (s *server) WatchEvent(r *api.WatchEventRequest, stream api.GobgpApi_WatchEventServer) error {
